// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2008-2010 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Net;

using Smdn.Net.Imap4.Client.Session;

namespace Smdn.Net.Imap4.WebClients {
  internal class ImapFetchMessageWebRequest : ImapMessageWebRequestBase {
    public ImapFetchMessageWebRequest(Uri requestUri, ImapSessionManager sessionManager)
      : base(requestUri, ImapWebRequestMethods.Fetch, sessionManager)
    {
    }

    private ImapSequenceSet RequestedUidSet {
      get { return ImapSequenceSet.FromUri(RequestUri); }
    }

    protected override ImapWebResponse InternalGetResponse()
    {
      var response = SelectRequestMailbox();

      if (response != null && response.Result.Failed)
        return response;

      switch (Method) {
        case ImapWebRequestMethods.Fetch:   return GetFetchResponse();
        case ImapWebRequestMethods.Expunge: return GetExpungeResponse(RequestedUidSet);
        case ImapWebRequestMethods.Copy:    return GetCopyResponse(RequestedUidSet);
        case ImapWebRequestMethods.Store:   return GetStoreResponse(RequestedUidSet);
        default: throw new NotSupportedException(string.Format("unsupported request method: {0}", Method));
      }
    }

    private ImapWebResponse GetFetchResponse()
    {
      ImapMessage[] messages;

      var response = new ImapWebResponse(Session.Fetch(RequestedUidSet,
                                                       ImapFetchDataItem.BodyStructure + ImapFetchDataItem.Rfc822Size,
                                                       out messages));

      if (response.Result.Failed)
        return response;
      else if (messages.Length == 0)
        throw new WebException("no such message", null, WebExceptionStatus.Success, response);

      var fetchMessage = messages[0];

      response.SetResponseStream(new ImapFetchMessageBodyStream(Session,
                                                                KeepAlive,
                                                                ImapStyleUriParser.GetUid(RequestUri),
                                                                ImapStyleUriParser.GetSection(RequestUri),
                                                                fetchMessage.Rfc822Size,
                                                                FetchBlockSize,
                                                                ReadWriteTimeout),
                                 fetchMessage.Rfc822Size,
                                 true);

      if (fetchMessage.BodyStructure != null) {
        if (fetchMessage.BodyStructure.IsMultiPart)
          response.SetContentType(string.Format("multipart/{0}",
                                                fetchMessage.BodyStructure.Subtype));
        else
          response.SetContentType(string.Format("{0}/{1}",
                                                (fetchMessage.BodyStructure as ImapSinglePartBodyStructure).Type,
                                                fetchMessage.BodyStructure.Subtype));
      }

      return response;
    }
  }
}
